@@ -37,13 +37,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
+#include "magick/profile.h"
 #include "magick/tempfile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
@@ -360,8 +361,8 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
     {
       for (i=0; i < (long) *bytes_per_line; i++)
       {
-        *q++=(*p >> 4) & 0xff;
-        *q++=(*p & 15);
+        *q++=(*p >> 4U) & 0xffU;
+        *q++=(*p & 15U);
         p++;
       }
       *bytes_per_line*=2;
@@ -371,10 +372,10 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
     {
       for (i=0; i < (long) *bytes_per_line; i++)
       {
-        *q++=(*p >> 6) & 0x03;
-        *q++=(*p >> 4) & 0x03;
-        *q++=(*p >> 2) & 0x03;
-        *q++=(*p & 3);
+        *q++=(*p >> 6U) & 0x03U;
+        *q++=(*p >> 4U) & 0x03U;
+        *q++=(*p >> 2U) & 0x03U;
+        *q++=(*p & 3U);
         p++;
       }
       *bytes_per_line*=4;
@@ -403,8 +404,9 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
   return(scanline);
 }
 
-static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
-  Image *image,unsigned long bytes_per_line,const unsigned int bits_per_pixel)
+static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
+  Image *blob,Image *image,unsigned long bytes_per_line,
+  const unsigned int bits_per_pixel)
 {
   long
     j,
@@ -418,15 +420,14 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
     *q;
 
   size_t
-    length;
+    allocated_pixels,
+    length,
+    row_bytes;
 
   unsigned char
     *pixels,
     *scanline;
 
-  unsigned short
-    row_bytes;
-
   unsigned long
     bytes_per_pixel,
     number_pixels,
@@ -450,16 +451,17 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
       width*=image->matte ? 4 : 3;
   if (bytes_per_line == 0)
     bytes_per_line=width;
-  row_bytes=(unsigned short) (image->columns | 0x8000);
+  row_bytes=(size_t) (image->columns | 0x8000);
   if (image->storage_class == DirectClass)
-    row_bytes=(unsigned short) ((4*image->columns) | 0x8000);
+    row_bytes=(size_t) ((4*image->columns) | 0x8000);
   /*
     Allocate pixel and scanline buffer.
   */
-  pixels=MagickAllocateMemory(unsigned char *,row_bytes*image->rows);
+  pixels=MagickAllocateArray(unsigned char *,image->rows,row_bytes);
   if (pixels == (unsigned char *) NULL)
     return((unsigned char *) NULL);
-  memset(pixels,0,row_bytes*image->rows);
+  allocated_pixels=image->rows*row_bytes;
+  (void) memset(pixels,0,allocated_pixels);
   scanline=MagickAllocateMemory(unsigned char *,row_bytes);
   if (scanline == (unsigned char *) NULL)
     return((unsigned char *) NULL);
@@ -491,7 +493,8 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
       scanline_length=ReadBlobByte(blob);
     if (scanline_length >= row_bytes)
       {
-        ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,"scanline length exceeds row bytes");
+        ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
+                       "scanline length exceeds row bytes");
         break;
       }
     (void) ReadBlob(blob,scanline_length,(char *) scanline);
@@ -501,6 +504,13 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
           length=(scanline[j] & 0xff)+1;
           number_pixels=length*bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
+          if ((q+number_pixels > pixels+allocated_pixels))
+            {
+              ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
+                             "Decoded RLE pixels exceeds allocation!");
+              break;
+            }
+
           (void) memcpy(q,p,number_pixels);
           q+=number_pixels;
           j+=length*bytes_per_pixel+1;
@@ -512,6 +522,12 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
           for (i=0; i < (long) length; i++)
           {
+          if ((q+number_pixels > pixels+allocated_pixels))
+            {
+              ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
+                             "Decoded RLE pixels exceeds allocation!");
+              break;
+            }
             (void) memcpy(q,p,number_pixels);
             q+=number_pixels;
           }
@@ -559,7 +575,7 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
 static size_t EncodeImage(Image *image,const unsigned char *scanline,
   const unsigned long bytes_per_line,unsigned char *pixels)
 {
-#define MaxCount  128
+#define MaxCount  128U
 #define MaxPackbitsRunlength  128
 
   long
@@ -683,7 +699,6 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   return(length);
 }
 
-#if !defined(macintosh)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -774,9 +789,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+  pixmap.bits_per_pixel=0;
+  pixmap.component_count=0;
   /*
     Read PICT header.
   */
+  pixmap.bits_per_pixel=0;
+  pixmap.component_count=0;
   for (i=0; i < 512; i++)
     (void) ReadBlobByte(image);  /* skip header */
   (void) ReadBlobMSBShort(image);  /* skip picture size */
@@ -794,12 +813,26 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   else
     if (version != 1)
       ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+  /*
+    Validate dimensions
+  */
+  if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) || (frame.bottom < 0) ||
+      (frame.left >= frame.right) || (frame.top >= frame.bottom))
+    {
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+    }
+
   /*
     Create black canvas.
   */
   flags=0;
   image->columns=frame.right-frame.left;
   image->rows=frame.bottom-frame.top;
+
+  if (IsEventLogging())
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Dimensions: %lux%lu",image->columns,image->rows);
+
   /*
     Interpret PICT opcodes.
   */
@@ -816,12 +849,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     if (code > 0xa1)
       {
         if (IsEventLogging())
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%04X:",code);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Code %04X:",code);
       }
     else
       {
         if (IsEventLogging())
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  %04X %.1024s: %.1024s",code,
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Code  %04X %.1024s: %.1024s",code,
             codes[code].name,codes[code].description);
         switch (code)
         {
@@ -842,7 +875,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               break;
             image->columns=frame.right-frame.left;
             image->rows=frame.bottom-frame.top;
-            SetImage(image,OpaqueOpacity);
+            (void) SetImage(image,OpaqueOpacity);
             break;
           }
           case 0x12:
@@ -1006,7 +1039,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
                   }
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                  "Allocated tile image colormap with %lu colors",tile_image->colors);
+                  "Allocated tile image colormap with %u colors",tile_image->colors);
                 if (bytes_per_line & 0x8000)
                   {
                     for (i=0; i < (long) tile_image->colors; i++)
@@ -1067,7 +1100,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               q=SetImagePixels(tile_image,0,y,tile_image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(tile_image);
+              indexes=AccessMutableIndexes(tile_image);
               for (x=0; x < (long) tile_image->columns; x++)
               {
                 if (tile_image->storage_class == PseudoClass)
@@ -1118,7 +1151,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 p+=(pixmap.component_count-1)*tile_image->columns;
               if (destination.bottom == (long) image->rows)
                 if (QuantumTick(y,tile_image->rows))
-                  if (!MagickMonitor(LoadImageText,y,tile_image->rows,&image->exception))
+                  if (!MagickMonitorFormatted(y,tile_image->rows,&image->exception,
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
             MagickFreeMemory(pixels);
@@ -1129,7 +1164,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                    destination.left,destination.top);
                 DestroyImage(tile_image);
             if (destination.bottom != (long) image->rows)
-              if (!MagickMonitor(LoadImageText,destination.bottom,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(destination.bottom,image->rows,&image->exception,
+                                          LoadImageText,image->filename,
+					  image->columns,image->rows))
                 break;
             break;
           }
@@ -1138,7 +1175,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             unsigned char
               *info;
 
-            unsigned long
+            unsigned int
               type;
 
             /*
@@ -1162,24 +1199,18 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 if (length == 0)
                   break;
-                image->color_profile.info=MagickAllocateMemory(unsigned char *,
-                  length);
-                if (image->color_profile.info == (unsigned char *) NULL)
+                if (SetImageProfile(image,"ICM",info,length) == MagickFail)
                   ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-                image->color_profile.length=length;
-                (void) memcpy(image->color_profile.info,info,length);
+                MagickFreeMemory(info);
                 break;
               }
               case 0x1f2:
               {
                 if (length == 0)
                   break;
-                image->iptc_profile.info=MagickAllocateMemory(unsigned char *,
-                  length);
-                if (image->iptc_profile.info == (unsigned char *) NULL)
+                if (SetImageProfile(image,"IPTC",info,length) == MagickFail)
                   ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-                image->iptc_profile.length=length;
-                (void) memcpy(image->iptc_profile.info,info,length);
+                MagickFreeMemory(info);
                 break;
               }
               default:
@@ -1250,7 +1281,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         }
         (void) fclose(file);
         tile_image=ReadImage(clone_info,exception);
-        LiberateTemporaryFile(clone_info->filename);
+        (void) LiberateTemporaryFile(clone_info->filename);
         DestroyImageInfo(clone_info);
         if (tile_image == (Image *) NULL)
           continue;
@@ -1293,7 +1324,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   CloseBlob(image);
   return(image);
 }
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1327,15 +1357,18 @@ ModuleExport void RegisterPICTImage(void)
   entry->decoder=(DecoderHandler) ReadPICTImage;
   entry->encoder=(EncoderHandler) WritePICTImage;
   entry->adjoin=False;
-  entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
-  entry->module=AcquireString("PICT");
+  entry->description="Apple Macintosh QuickDraw/PICT";
+  entry->seekable_stream=MagickTrue;
+  entry->module="PICT";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("PICT");
   entry->decoder=(DecoderHandler) ReadPICTImage;
   entry->encoder=(EncoderHandler) WritePICTImage;
   entry->adjoin=False;
-  entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
-  entry->module=AcquireString("PICT");
+  entry->description="Apple Macintosh QuickDraw/PICT";
+  entry->seekable_stream=MagickTrue;
+  entry->module="PICT";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1396,7 +1429,7 @@ ModuleExport void UnregisterPICTImage(void)
 */
 static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
 {
-#define MaxCount  128
+#define MaxCount  128U
 #define PictCropRegionOp  0x01
 #define PictEndOfPictureOp  0xff
 #define PictJPEGOp  0x8200
@@ -1427,10 +1460,16 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     size_rectangle,
     source_rectangle;
 
+  const unsigned char
+    *profile_info;
+  
+  size_t
+    profile_length;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -1469,7 +1508,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
-  TransformColorspace(image,RGBColorspace);
+  (void) TransformColorspace(image,RGBColorspace);
   /*
     Initialize image info.
   */
@@ -1528,7 +1567,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
-  memset(buffer,0,PictInfoSize);
+  (void) memset(buffer,0,PictInfoSize);
   (void) WriteBlob(image,PictInfoSize,buffer);
   (void) WriteBlobMSBShort(image,0);
   (void) WriteBlobMSBShort(image,size_rectangle.top);
@@ -1551,23 +1590,31 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBShort(image,frame_rectangle.bottom);
   (void) WriteBlobMSBShort(image,frame_rectangle.right);
   (void) WriteBlobMSBLong(image,0x00000000L);
-  if (image->iptc_profile.info != (unsigned char *) NULL)
+  /*
+    Output 8BIM profile.
+  */
+  profile_info=GetImageProfile(image,"8BIM",&profile_length);
+  if (profile_info != (unsigned char *) NULL)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0x1f2);
-      (void) WriteBlobMSBShort(image,image->iptc_profile.length+4);
+      (void) WriteBlobMSBShort(image,profile_length+4);
       (void) WriteBlobString(image,"8BIM");
-      (void) WriteBlob(image,image->iptc_profile.length,
-        image->iptc_profile.info);
+      (void) WriteBlob(image,profile_length,
+                       profile_info);
     }
-  if (image->color_profile.info != (unsigned char *) NULL)
+  /*
+    Ouput ICM profile.
+  */
+  profile_info=GetImageProfile(image,"ICM",&profile_length);
+  if (profile_info != (unsigned char *) NULL)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
-      (void) WriteBlobMSBShort(image,image->color_profile.length+4);
+      (void) WriteBlobMSBShort(image,profile_length+4);
       (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlob(image,image->color_profile.length,
-        image->color_profile.info);
+      (void) WriteBlob(image,profile_length,
+                       profile_info);
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,4);
@@ -1735,12 +1782,14 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=AccessImmutableIndexes(image);
       for (x=0; x < (long) image->columns; x++)
         scanline[x]=indexes[x];
       count+=EncodeImage(image,scanline,row_bytes & 0x7FFF,packed_scanline);
       if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+        if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                    SaveImageText,image->filename,
+				    image->columns,image->rows))
           break;
     }
   else
@@ -1788,7 +1837,9 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
           }
           count+=EncodeImage(image,scanline,bytes_per_line & 0x7FFF,packed_scanline);
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+            if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
               break;
         }
       }
