@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2008 - 2009 GraphicsMagick Group
+% Copyright (C) 2008 - 2012 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -521,6 +521,18 @@ MagickExport const char* CompressionTypeToString(const CompressionType compressi
     case ZipCompression:
       compression_string="Zip";
       break;
+    case LZMACompression:
+      compression_string="LZMA";
+      break;
+    case JPEG2000Compression:
+      compression_string="JPEG2000";
+      break;
+    case JBIG1Compression:
+      compression_string="JBIG1";
+      break;
+    case JBIG2Compression:
+      compression_string="JBIG2";
+      break;
     }
   return compression_string;
 }
@@ -539,6 +551,8 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=BZipCompression;
   else if (LocaleCompare("Fax",option) == 0)
     compression_type=FaxCompression;
+  else if (LocaleCompare("Group3",option) == 0)
+    compression_type=Group3Compression;
   else if (LocaleCompare("Group4",option) == 0)
     compression_type=Group4Compression;
   else if (LocaleCompare("JPEG",option) == 0)
@@ -553,6 +567,16 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=ZipCompression;
   else if (LocaleCompare("GZip",option) == 0)
     compression_type=ZipCompression;
+  else if ((LocaleCompare("LZMA",option) == 0) ||
+	   (LocaleCompare("LZMA2",option) == 0))
+    compression_type=LZMACompression;
+  else if (LocaleCompare("JPEG2000",option) == 0)
+    compression_type=JPEG2000Compression;
+  else if ((LocaleCompare("JBIG",option) == 0) ||
+	   (LocaleCompare("JBIG1",option) == 0))
+    compression_type=JBIG1Compression;
+  else if (LocaleCompare("JBIG2",option) == 0)
+    compression_type=JBIG2Compression;
 
   return compression_type;
 }
@@ -964,6 +988,68 @@ MagickExport MetricType StringToMetricType(const char *option)
   return metric_type;
 }
 
+/*
+  NoiseType
+*/
+MagickExport const char *NoiseTypeToString(NoiseType noise_type)
+{
+  char
+    *noise_string="Undefined";
+
+  switch (noise_type)
+    {
+    case UniformNoise:
+      noise_string="Uniform";
+      break;
+    case GaussianNoise:
+      noise_string="Gaussian";
+      break;
+    case MultiplicativeGaussianNoise:
+      noise_string="Multiplicative";
+      break;
+    case ImpulseNoise:
+      noise_string="Impulse";
+      break;
+    case LaplacianNoise:
+      noise_string="Laplacian";
+      break;
+    case PoissonNoise:
+      noise_string="Poisson";
+      break;
+    case RandomNoise:
+      noise_string="Random";
+      break;
+    case UndefinedNoise:
+      noise_string="Undefined";
+      break;
+    }
+
+  return noise_string;
+}
+MagickExport NoiseType StringToNoiseType(const char *option)
+{
+  NoiseType
+    noise_type=UndefinedNoise;
+
+  if (LocaleCompare("Uniform",option) == 0)
+    noise_type=UniformNoise;
+  else if (LocaleCompare("Gaussian",option) == 0)
+    noise_type=GaussianNoise;
+  else if (LocaleCompare("Multiplicative",option) == 0)
+    noise_type=MultiplicativeGaussianNoise;
+  else if (LocaleCompare("Impulse",option) == 0)
+    noise_type=ImpulseNoise;
+  else if (LocaleCompare("Laplacian",option) == 0)
+    noise_type=LaplacianNoise;
+  else if (LocaleCompare("Poisson",option) == 0)
+    noise_type=PoissonNoise;
+  else if (LocaleCompare("Random",option) == 0)
+    noise_type=RandomNoise;
+  else if (LocaleCompare("Undefined",option) == 0)
+    noise_type=UndefinedNoise;
+  return noise_type;
+}
+
 /*
   OrientationType
 */
@@ -1132,6 +1218,9 @@ MagickExport QuantumOperator StringToQuantumOperator(const char *option)
   else if ((LocaleCompare("noise-poisson",option) == 0) ||
            (LocaleCompare("PoissonNoise",option) == 0))
     quantum_operator=NoisePoissonQuantumOp;
+  else if ((LocaleCompare("noise-random",option) == 0) ||
+           (LocaleCompare("RandomNoise",option) == 0))
+    quantum_operator=NoiseRandomQuantumOp;
   else if ((LocaleCompare("noise-uniform",option) == 0) ||
            (LocaleCompare("UniformNoise",option) == 0))
     quantum_operator=NoiseUniformQuantumOp;
@@ -1240,6 +1329,9 @@ MagickExport const char *QuantumOperatorToString(const QuantumOperator quantum_o
     case PowQuantumOp:
       operator_text="pow";
       break;
+    case NoiseRandomQuantumOp:
+      operator_text="noise-random";
+      break;
     }
 
   return operator_text;
