@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2004 - 2008 GraphicsMagick Group
+% Copyright (C) 2004 - 2012 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -17,6 +17,7 @@
 #include "magick/enum_strings.h"
 #include "magick/gem.h"
 #include "magick/pixel_iterator.h"
+#include "magick/random.h"
 #include "magick/utility.h"
 #include "magick/operator.h"
 
@@ -89,9 +90,10 @@ typedef struct _ChannelOptions_t
 %        LShiftQuantumOp, MultiplyQuantumOp,  NegateQuantumOp,
 %        NoiseGaussianQuantumOp, NoiseImpulseQuantumOp,
 %        NoiseLaplacianQuantumOp, NoiseMultiplicativeQuantumOp,
-%        NoisePoissonQuantumOp, NoiseUniformQuantumOp, OrQuantumOp,
-%        RShiftQuantumOp, SubtractQuantumOp, ThresholdBlackQuantumOp,
-%        ThresholdQuantumOp, ThresholdWhiteQuantumOp, XorQuantumOp).
+%        NoisePoissonQuantumOp, NoiseRandomQuantumOp, NoiseUniformQuantumOp,
+%        OrQuantumOp, RShiftQuantumOp, SubtractQuantumOp,
+%        ThresholdBlackQuantumOp, ThresholdQuantumOp, ThresholdWhiteQuantumOp,
+%        XorQuantumOp).
 %
 %    o rvalue: Operator argument.
 %
@@ -300,9 +302,10 @@ QuantumOperatorImageMultivalue(Image *image,
 %        LShiftQuantumOp, MultiplyQuantumOp,  NegateQuantumOp,
 %        NoiseGaussianQuantumOp, NoiseImpulseQuantumOp,
 %        NoiseLaplacianQuantumOp, NoiseMultiplicativeQuantumOp,
-%        NoisePoissonQuantumOp, NoiseUniformQuantumOp, OrQuantumOp,
-%        RShiftQuantumOp, SubtractQuantumOp, ThresholdBlackQuantumOp,
-%        ThresholdQuantumOp, ThresholdWhiteQuantumOp, XorQuantumOp).
+%        NoisePoissonQuantumOp, NoiseRandomQuantumOp, NoiseUniformQuantumOp,
+%        OrQuantumOp, RShiftQuantumOp, SubtractQuantumOp,
+%        ThresholdBlackQuantumOp, ThresholdQuantumOp, ThresholdWhiteQuantumOp,
+%        XorQuantumOp).
 %
 %    o rvalue: Operator argument.
 %
@@ -571,7 +574,7 @@ QuantumDepthCB(void *mutable_data,
       */
 #if MaxRGB <= MaxMap
 #  if defined(HAVE_OPENMP)
-#    pragma omp critical
+#    pragma omp critical (GM_QuantumDepthCB)
 #  endif
       if (mutable_context->channel_lut == (Quantum *) NULL)
         {
@@ -751,7 +754,7 @@ QuantumGammaCB(void *mutable_data,
   */
 #if MaxRGB <= MaxMap
 #  if defined(HAVE_OPENMP)
-#    pragma omp critical
+#    pragma omp critical (GM_QuantumGammaCB)
 #  endif
   if (mutable_context->channel_lut == (Quantum *) NULL)
     {
@@ -925,7 +928,7 @@ QuantumLogCB(void *mutable_data,
   */
 #if MaxRGB <= MaxMap
 #  if defined(HAVE_OPENMP)
-#    pragma omp critical
+#    pragma omp critical (GM_QuantumLogCB)
 #  endif
   if (mutable_context->channel_lut == (Quantum *) NULL)
     {
@@ -1287,13 +1290,13 @@ QuantumMultiplyCB(void *mutable_data,
 
 static inline Quantum
 GenerateQuantumNoise(const Quantum quantum,const NoiseType noise_type,
-                     const double factor,unsigned int *seed)
+                     const double factor,MagickRandomKernel *kernel)
 {
   double
     value;
 
   value = (double) quantum+
-    factor*GenerateDifferentialNoise((double) quantum,noise_type,seed);
+    factor*GenerateDifferentialNoise((double) quantum,noise_type,kernel);
   return RoundDoubleToQuantum(value);
 }
 
@@ -1317,15 +1320,15 @@ QuantumNoiseCB(void *mutable_data,
   double
     factor;
 
-  unsigned int
-    seed;
+  MagickRandomKernel
+    *kernel;
 
   ARG_NOT_USED(mutable_data);
   ARG_NOT_USED(image);
   ARG_NOT_USED(indexes);
   ARG_NOT_USED(exception);
 
-  seed=MagickRandNewSeed();
+  kernel=AcquireMagickRandomKernel();
   factor=context->double_value/MaxRGBDouble;
 
   switch (context->channel)
@@ -1333,31 +1336,31 @@ QuantumNoiseCB(void *mutable_data,
     case RedChannel:
     case CyanChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].red = GenerateQuantumNoise(pixels[i].red,noise_type,factor,&seed);
+        pixels[i].red = GenerateQuantumNoise(pixels[i].red,noise_type,factor,kernel);
       break;
     case GreenChannel:
     case MagentaChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,&seed);
+        pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,kernel);
       break;
     case BlueChannel:
     case YellowChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].blue = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,&seed);
+        pixels[i].blue = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,kernel);
       break;
     case BlackChannel:
     case MatteChannel:
     case OpacityChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].opacity = GenerateQuantumNoise(pixels[i].opacity,noise_type,factor,&seed);
+        pixels[i].opacity = GenerateQuantumNoise(pixels[i].opacity,noise_type,factor,kernel);
       break;
     case UndefinedChannel:
     case AllChannels:
       for (i=0; i < npixels; i++)
         {
-          pixels[i].red   = GenerateQuantumNoise(pixels[i].red,noise_type,factor,&seed);
-          pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,&seed);
-          pixels[i].blue  = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,&seed);
+          pixels[i].red   = GenerateQuantumNoise(pixels[i].red,noise_type,factor,kernel);
+          pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,kernel);
+          pixels[i].blue  = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,kernel);
         }
       break;
     case GrayChannel:
@@ -1368,7 +1371,7 @@ QuantumNoiseCB(void *mutable_data,
 
           intensity = PixelIntensity(&pixels[i]);
           pixels[i].red = pixels[i].green = pixels[i].blue =
-            GenerateQuantumNoise(intensity,noise_type,factor,&seed);
+            GenerateQuantumNoise(intensity,noise_type,factor,kernel);
         }
       break;
     }
@@ -1436,6 +1439,21 @@ QuantumNoisePoissonCB(void *mutable_data,
   return
     QuantumNoiseCB(mutable_data,immutable_data,image,pixels,indexes,npixels,exception,PoissonNoise);
 }
+
+
+static MagickPassFail
+QuantumNoiseRandomCB(void *mutable_data,
+		     const void *immutable_data,
+                      Image *image,
+                      PixelPacket *pixels,
+                      IndexPacket *indexes,
+                      const long npixels,
+                      ExceptionInfo *exception)
+{
+  return
+    QuantumNoiseCB(mutable_data,immutable_data,image,pixels,indexes,npixels,exception,RandomNoise);
+}
+
 static MagickPassFail
 QuantumNoiseUniformCB(void *mutable_data,
                       const void *immutable_data,
@@ -1550,7 +1568,7 @@ QuantumPowCB(void *mutable_data,
   */
 #if MaxRGB <= MaxMap
 #  if defined(HAVE_OPENMP)
-#    pragma omp critical
+#    pragma omp critical (GM_QuantumPowCB)
 #  endif
   if (mutable_context->channel_lut == (Quantum *) NULL)
     {
@@ -2192,11 +2210,14 @@ QuantumOperatorRegionImage(Image *image,
     case PowQuantumOp:
       call_back=QuantumPowCB;
       break;
+    case NoiseRandomQuantumOp:
+      call_back=QuantumNoiseRandomCB;
+      break;
     }
 
   if (call_back)
     {
-      FormatString(description,"[%%s] Apply operator '%s %g (%g%%)' to channel '%s'...",
+      FormatString(description,"[%%s] Apply operator '%s %g (%g%%%%)' to channel '%s'...",
                    QuantumOperatorToString(quantum_operator),rvalue,
                    ((rvalue/MaxRGBFloat)*100),
                    ChannelTypeToString(channel));
