@@ -45,6 +45,7 @@
 #include "magick/alpha_composite.h"
 #include "magick/color.h"
 #include "magick/decorate.h"
+#include "magick/log.h"
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/render.h"
@@ -416,7 +417,11 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
 
 #if defined(IntegralRotateImageUseOpenMP)
 #  if defined(HAVE_OPENMP)
-#    pragma omp parallel for schedule(static,1) shared(status, tile)
+#    if defined(TUNE_OPENMP)
+#      pragma omp parallel for schedule(runtime) shared(status, tile)
+#    else
+#      pragma omp parallel for schedule(static,1) shared(status, tile)
+#    endif
 #  endif
 #endif
         for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height_max)
@@ -427,6 +432,11 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
             MagickPassFail
               thread_status;
 
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_IntegralRotateImage)
+#  endif
+#endif
             thread_status=status;
             if (thread_status == MagickFail)
               continue;
@@ -578,7 +588,11 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
         (void) strlcpy(message,"[%s] Rotate: 180 degrees...",sizeof(message));
 #if defined(IntegralRotateImageUseOpenMP)
 #  if defined(HAVE_OPENMP)
-#    pragma omp parallel for schedule(static,8) shared(row_count, status)
+#    if defined(TUNE_OPENMP)
+#      pragma omp parallel for schedule(runtime) shared(row_count, status)
+#    else
+#      pragma omp parallel for schedule(static,8) shared(row_count, status)
+#    endif
 #  endif
 #endif
         for (y=0; y < (long) image->rows; y++)
@@ -601,6 +615,11 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
             MagickPassFail
               thread_status;
 
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_IntegralRotateImage)
+#  endif
+#endif
             thread_status=status;
             if (thread_status == MagickFail)
               continue;
@@ -665,7 +684,11 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
         tile=0;
 #if defined(IntegralRotateImageUseOpenMP)
 #  if defined(HAVE_OPENMP)
-#    pragma omp parallel for schedule(static,1) shared(status, tile)
+#    if defined(TUNE_OPENMP)
+#      pragma omp parallel for schedule(runtime) shared(status, tile)
+#    else
+#      pragma omp parallel for schedule(static,1) shared(status, tile)
+#    endif
 #  endif
 #endif
         for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height_max)
@@ -676,6 +699,11 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
             MagickPassFail
               thread_status;
 
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_IntegralRotateImage)
+#  endif
+#endif
             thread_status=status;
             if (thread_status == MagickFail)
               continue;
@@ -802,13 +830,13 @@ IntegralRotateImage(const Image *image,unsigned int rotations,
                     if (!MagickMonitorFormatted(tile,total_tiles,exception,
                                                 message,image->filename))
                       thread_status=MagickFail;
+
+		  if (thread_status == MagickFail)
+		    status=MagickFail;
                 }
 
                 if (thread_status == MagickFail)
-                  {
-                    status=MagickFail;
-                    break;
-                  }
+		  break;
               }
           }
         Swap(page.width,page.height);
@@ -883,7 +911,15 @@ XShearImage(Image *image,const double degrees,
   is_grayscale=image->is_grayscale;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,8) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    if defined(USE_STATIC_SCHEDULING_ONLY)
+#      pragma omp parallel for schedule(static) shared(row_count, status)
+#    else
+#      pragma omp parallel for schedule(dynamic) shared(row_count, status)
+#    endif
+#  endif
 #endif
   for (y=0; y < (long) height; y++)
     {
@@ -912,7 +948,10 @@ XShearImage(Image *image,const double degrees,
 
       MagickPassFail
         thread_status;
-      
+
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_XShearImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -1150,7 +1189,15 @@ YShearImage(Image *image,const double degrees,
   is_grayscale=image->is_grayscale;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,8) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    if defined(USE_STATIC_SCHEDULING_ONLY)
+#      pragma omp parallel for schedule(static) shared(row_count, status)
+#    else
+#      pragma omp parallel for schedule(dynamic) shared(row_count, status)
+#    endif
+#  endif
 #endif
   for (y=0; y < (long) width; y++)
     {
@@ -1179,7 +1226,10 @@ YShearImage(Image *image,const double degrees,
 
       MagickPassFail
         thread_status;
-      
+
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_YShearImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -1594,7 +1644,9 @@ ShearImage(const Image *image,const double x_shear,
     goto shear_image_exception;
   shear.x=(-tan(DegreesToRadians(x_shear)/2.0));
   shear.y=sin(DegreesToRadians(y_shear));
-  if ((shear.x == 0.0) || (shear.y == 0.0))
+  (void) LogMagickEvent(TransformEvent,GetMagickModule(),
+			"Shear angles x,y: %g,%g degrees", shear.x, shear.y);
+  if ((shear.x == 0.0) && (shear.y == 0.0))
     return(integral_image);
 
   /*
