@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2008 GraphicsMagick Group
+% Copyright (C) 2003 - 2010 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -37,11 +37,13 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/analyze.h"
 #include "magick/color.h"
 #include "magick/composite.h"
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/resize.h"
+#include "magick/texture.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
 #include "magick/log.h"
@@ -78,7 +80,7 @@
 MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
                               ExceptionInfo *exception)
 {
-#define ChopImageText  "[%s] Chop image..."
+#define ChopImageText "[%s] Chop..."
 
   Image
     *chop_image;
@@ -136,7 +138,11 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     Extract chop image.
   */
 #if defined(HAVE_OPENMP)  && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    pragma omp parallel for schedule(static,4) shared(row_count, status)
+#  endif
 #endif
   for (y=0; y < (long) clone_info.y; y++)
     {
@@ -158,6 +164,9 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       MagickBool
         thread_status;
 
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_ChopImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -187,7 +196,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
             thread_status=MagickFail;
         }
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ChopImage)
 #endif
       {
         row_count++;
@@ -204,7 +213,11 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     Extract chop image.
   */
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    pragma omp parallel for schedule(static,4) shared(row_count, status)
+#  endif
 #endif
   for (y=0; y < (long) (image->rows-(clone_info.y+clone_info.height)); y++)
     {
@@ -226,6 +239,9 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       MagickBool
         thread_status;
 
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_ChopImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -255,7 +271,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
             thread_status=MagickFail;
         }
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ChopImage)
 #endif
       {
         row_count++;
@@ -281,6 +297,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %     C o a l e s c e I m a g e s                                             %
 %                                                                             %
 %                                                                             %
@@ -314,6 +331,12 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
   register const Image
     *next;
 
+  register long
+    i;
+  
+  MagickBool
+    found_transparency=False;
+
   /*
     Coalesce the image sequence.
   */
@@ -349,9 +372,21 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
       }
       case BackgroundDispose:
       {
+        /*
+	  Fill image with transparent color, if one exists.
+	*/
         coalesce_image->next=CloneImage(coalesce_image,0,0,True,exception);
-        if (coalesce_image->next != (Image *) NULL)
-          (void) SetImage(coalesce_image->next,OpaqueOpacity);
+        if (coalesce_image->next != (Image *) NULL) {
+          for (i = 0; i < (long) coalesce_image->colors; i++) {
+            if (coalesce_image->colormap[i].opacity == TransparentOpacity) {
+              found_transparency = True;
+              (void) SetImageColor(coalesce_image->next,&coalesce_image->colormap[i]);
+              break;
+            }
+          }
+          if (!found_transparency)
+            (void) SetImage(coalesce_image->next,OpaqueOpacity);
+        }
         break;
       }
       case PreviousDispose:
@@ -373,6 +408,7 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
     (void) CompositeImage(coalesce_image,next->matte ? OverCompositeOp :
       CopyCompositeOp,next,next->page.x,next->page.y);
   }
+
   while (coalesce_image->previous != (Image *) NULL)
     coalesce_image=coalesce_image->previous;
   return(coalesce_image);
@@ -501,7 +537,11 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   if ((geometry->width == 0) || (geometry->height == 0))
     (void) memset(&crop_image->page,0,sizeof(RectangleInfo));
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    pragma omp parallel for schedule(static,4) shared(row_count, status)
+#  endif
 #endif
   for (y=0; y < (long) crop_image->rows; y++)
     {
@@ -520,6 +560,9 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
       MagickBool
         thread_status;
 
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_CropImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -542,13 +585,13 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
             thread_status=MagickFail;
         }
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_CropImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,crop_image->rows))
-          if (!MagickMonitorFormatted(row_count,crop_image->rows-1,exception,
-                                      "[%s] Crop %lux%lu+%ld+%ld image...",
+          if (!MagickMonitorFormatted(row_count,crop_image->rows,exception,
+                                      "[%s] Crop: %lux%lu+%ld+%ld...",
                                       crop_image->filename,
                                       crop_image->columns,crop_image->rows,
                                       page.x,page.y))
@@ -571,6 +614,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %     D e c o n s t r u c t I m a g e s                                       %
 %                                                                             %
 %                                                                             %
@@ -763,6 +807,85 @@ MagickExport Image *DeconstructImages(const Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
+%   E x t e n t I m a g e                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Use ExtentImage() to change the image dimensions as specified by geometry
+%  width and height.  The existing image content is composited at the position
+%  specified by geometry x and y using the image compose method.  Existing
+%  image content which falls outside the bounds of the new image dimensions
+%  is discarded.
+%
+%  The format of the ExtentImage method is:
+%
+%      Image *ExtentImage(const Image *image,const RectangleInfo *geometry,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o geometry: Define the new image dimension with width and height, and
+%        the top left coordinate to place the existing image content with
+%        x and y.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport Image *ExtentImage(const Image *image,const RectangleInfo *geometry,
+				ExceptionInfo *exception)
+{
+  Image
+    *extent_image;
+
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  assert(geometry != (const RectangleInfo *) NULL);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+
+  /*
+    Allocate canvas image
+  */
+  if ((extent_image=CloneImage(image,geometry->width,geometry->height,
+			       MagickTrue,exception)) == (Image *) NULL)
+    return((Image *) NULL);
+
+  /*
+    Set canvas image color to background color
+  */
+  if ((SetImage(extent_image,image->background_color.opacity)) == MagickFail)
+    {
+      CopyException(exception,&extent_image->exception);
+      DestroyImage(extent_image);
+      return((Image *) NULL);
+    }
+
+  /*
+    Composite existing image at position using requested composition
+    operator.
+  */
+  if ((CompositeImage(extent_image,image->compose,image,geometry->x,
+		      geometry->y)) == MagickFail)
+    {
+      CopyException(exception,&extent_image->exception);
+      DestroyImage(extent_image);
+      return((Image *) NULL);
+    }
+
+  return(extent_image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %     F l a t t e n I m a g e                                                 %
 %                                                                             %
 %                                                                             %
@@ -798,21 +921,30 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
   assert(image->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (image->next == (Image *) NULL)
-    ThrowImageException3(ImageError,ImageSequenceIsRequired,
-      UnableToFlattenImage);
+
   /*
-    Clone first image in sequence.
+    Clone first image in sequence to serve as canvas image
   */
   flatten_image=CloneImage(image,0,0,True,exception);
-  if (flatten_image == (Image *) NULL)
-    return((Image *) NULL);
+
   /*
-    Flatten image.
+    Apply background color under image if it has a matte channel.
   */
-  for (next=image->next; next != (Image *) NULL; next=next->next)
-    (void) CompositeImage(flatten_image,next->compose,next,next->page.x,
-      next->page.y);
+  if ((flatten_image != (Image *) NULL) && (flatten_image->matte))
+    (void) MagickCompositeImageUnderColor(flatten_image,
+					  &flatten_image->background_color,
+					  exception);
+
+  if ((flatten_image != (Image *) NULL) &&
+      (image->next != (Image *) NULL))
+    {
+      /*
+	Flatten remaining images onto canvas
+      */
+      for (next=image->next; next != (Image *) NULL; next=next->next)
+	(void) CompositeImage(flatten_image,next->compose,next,next->page.x,
+			      next->page.y);
+    }
   return(flatten_image);
 }
 
@@ -844,7 +976,7 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 {
-#define FlipImageText  "[%s] Flip image..."
+#define FlipImageText "[%s] Flip..."
 
   Image
     *flip_image;
@@ -872,7 +1004,11 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
     Flip each row.
   */
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    pragma omp parallel for schedule(static,4) shared(row_count, status)
+#  endif
 #endif
   for (y=0; y < (long) flip_image->rows; y++)
     {
@@ -891,6 +1027,9 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
       MagickBool
         thread_status;
 
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_FlipImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -913,7 +1052,7 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
             thread_status=MagickFail;
         }
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_FlipImage)
 #endif
       {
         row_count++;
@@ -963,7 +1102,7 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 {
-#define FlopImageText  "[%s] Flop image..."
+#define FlopImageText "[%s] Flop..."
 
   Image
     *flop_image;
@@ -991,7 +1130,11 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
     Flop each row.
   */
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime) shared(row_count, status)
+#  else
+#    pragma omp parallel for schedule(static,4) shared(row_count, status)
+#  endif
 #endif
   for (y=0; y < (long) flop_image->rows; y++)
     {
@@ -1013,6 +1156,9 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
       MagickBool
         thread_status;
 
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_FlopImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -1040,7 +1186,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
             thread_status=MagickFail;
         }
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_FlopImage)
 #endif
       {
         row_count++;
@@ -1091,7 +1237,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
 {
-#define MosaicImageText  "[%s] Create image mosaic..."
+#define MosaicImageText "[%s] Create mosaic..."
 
   Image
     *mosaic_image;
@@ -1103,19 +1249,22 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
     *next;
 
   unsigned int
-    scene,
+    scene;
+
+  MagickBool
+    matte;
+
+  MagickPassFail
     status;
 
-  /*
-    Determine mosaic bounding box.
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (image->next == (Image *) NULL)
-    ThrowImageException3(ImageError,ImageSequenceIsRequired,
-      UnableToCreateImageMosaic);
+
+  /*
+    Determine mosaic bounding box.
+  */
   page.width=image->columns;
   page.height=image->rows;
   page.x=0;
@@ -1133,26 +1282,44 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
     if (next->page.height > page.height)
       page.height=next->page.height;
   }
+
   /*
-    Allocate mosaic image.
+    Allocate canvas image.
   */
   mosaic_image=AllocateImage((ImageInfo *) NULL);
   if (mosaic_image == (Image *) NULL)
     return((Image *) NULL);
   mosaic_image->columns=page.width;
   mosaic_image->rows=page.height;
-  (void) SetImage(mosaic_image,OpaqueOpacity);
+
   /*
-    Initialize colormap.
+    Canvas image supports transparency if any subordinate image uses
+    transparency.
+  */
+  matte=MagickTrue;
+  for (next=image; next != (Image *) NULL; next=next->next)
+    matte &= next->matte;
+  mosaic_image->matte=matte;
+
+  /*
+    Canvas color is copied from background color of first image in
+    list.  Default canvas color is 'white' but opaque 'black' or
+    'transparent' is often best for composition.
+  */
+  mosaic_image->background_color=image->background_color;
+  (void) SetImage(mosaic_image,OpaqueOpacity);
+
+  /*
+    Composite mosaic.
   */
   scene=0;
   for (next=image; next != (Image *) NULL; next=next->next)
   {
-    (void) CompositeImage(mosaic_image,CopyCompositeOp,next,next->page.x,
+    (void) CompositeImage(mosaic_image,next->compose,next,next->page.x,
       next->page.y);
     status=MagickMonitorFormatted(scene++,GetImageListLength(image),
                                   exception,MosaicImageText,image->filename);
-    if (status == False)
+    if (status == MagickFail)
       break;
   }
   return(mosaic_image);
@@ -1332,18 +1499,19 @@ MagickExport Image *ShaveImage(const Image *image,
 %
 */
 MagickExport void TransformImage(Image **image,const char *crop_geometry,
-  const char *image_geometry)
+				 const char *image_geometry)
 {
   Image
+    *previous,
     *resize_image,
     *transform_image;
 
-  int
-    flags;
-
   RectangleInfo
     geometry;
 
+  int
+    flags;
+
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickSignature);
   transform_image=(*image);
@@ -1352,9 +1520,6 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
       Image
         *crop_image;
 
-      RectangleInfo
-        geometry;
-
       /*
         Crop image to a user specified size.
       */
@@ -1363,7 +1528,9 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
       if ((geometry.width == 0) || (geometry.height == 0) ||
           ((flags & XValue) != 0) || ((flags & YValue) != 0) ||
           (flags & PercentValue))
-        crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
+	{
+	  crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
+	}
       else
         if ((transform_image->columns > geometry.width) ||
             (transform_image->rows > geometry.height))
@@ -1386,56 +1553,65 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
             height=geometry.height;
             next=(Image *) NULL;
             for (y=0; y < (long) transform_image->rows; y+=height)
-            {
-              for (x=0; x < (long) transform_image->columns; x+=width)
-              {
-                geometry.width=width;
-                geometry.height=height;
-                geometry.x=x;
-                geometry.y=y;
-                next=CropImage(transform_image,&geometry,&(*image)->exception);
-                if (next == (Image *) NULL)
-                  break;
-                if (crop_image == (Image *) NULL)
-                  crop_image=next;
-                else
-                  {
-                    next->previous=crop_image;
-                    crop_image->next=next;
-                    crop_image=crop_image->next;
-                  }
-              }
-              if (next == (Image *) NULL)
-                break;
-            }
+	      {
+		for (x=0; x < (long) transform_image->columns; x+=width)
+		  {
+		    geometry.width=width;
+		    geometry.height=height;
+		    geometry.x=x;
+		    geometry.y=y;
+		    next=CropImage(transform_image,&geometry,&(*image)->exception);
+		    if (next == (Image *) NULL)
+		      break;
+		    if (crop_image == (Image *) NULL)
+		      crop_image=next;
+		    else
+		      {
+			next->previous=crop_image;
+			crop_image->next=next;
+			crop_image=crop_image->next;
+		      }
+		  }
+		if (next == (Image *) NULL)
+		  break;
+	      }
           }
       if (crop_image != (Image *) NULL)
         {
+	  previous=transform_image->previous;
+	  crop_image->next=transform_image->next;
           DestroyImage(transform_image);
+	  transform_image=(Image *) NULL;
           while (crop_image->previous != (Image *) NULL)
             crop_image=crop_image->previous;
+	  crop_image->previous=previous;
           transform_image=crop_image;
         }
       *image=transform_image;
     }
   if (image_geometry == (const char *) NULL)
     return;
+
   /*
     Scale image to a user specified size.
   */
   SetGeometry(transform_image,&geometry);
   flags=GetMagickGeometry(image_geometry,&geometry.x,&geometry.y,
-    &geometry.width,&geometry.height);
+			  &geometry.width,&geometry.height);
   if ((transform_image->columns == geometry.width) &&
       (transform_image->rows == geometry.height))
     return;
+  
   /*
     Resize image.
   */
   resize_image=ZoomImage(transform_image,geometry.width,geometry.height,
-    &(*image)->exception);
+			 &(*image)->exception);
   if (resize_image == (Image *) NULL)
     return;
+
+  previous=transform_image->previous;
+  resize_image->next=transform_image->next;
   DestroyImage(transform_image);
   transform_image=resize_image;
   *image=transform_image;
