@@ -5,17 +5,29 @@
 # observe Bourne shell syntax.
 #
 
-# Package base name
-PACKAGE_NAME='GraphicsMagick'
 #
+# Package base name
+#
+PACKAGE_NAME='GraphicsMagick'
+
+#
+# Package bugs/support mailing list
+#
+PACKAGE_BUGREPORT='graphicsmagick-bugs@lists.sourceforge.net'
+
 # Package base version.  This is is the numeric version suffix applied to
-# PACKAGE_NAME (e.g. "1.3.5").
-PACKAGE_VERSION='1.3.5'
+# PACKAGE_NAME (e.g. "1.2").
+PACKAGE_VERSION='1.3.16'
+
+#
+# Package name plus version string.
+#
+PACKAGE_STRING="$PACKAGE_NAME $PACKAGE_VERSION"
 
 #
 # Formal Package release date
 # Set to string "unreleased" if package is not a formal release.
-PACKAGE_RELEASE_DATE="2009-01-26"
+PACKAGE_RELEASE_DATE="2012-06-23"
 #PACKAGE_RELEASE_DATE="unreleased"
 
 #
@@ -30,20 +42,31 @@ PACKAGE_CHANGE_DATE=`awk '/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/ { print
 if test "$PACKAGE_RELEASE_DATE" = "unreleased"
 then
   PACKAGE_VERSION_ADDENDUM=".0${PACKAGE_CHANGE_DATE}"
+  PACKAGE_RELEASE_DATE="snapshot-${PACKAGE_CHANGE_DATE}"
 else
   PACKAGE_VERSION_ADDENDUM=''
 fi
 
 #
-# CVS branch that this release is on.
+# Mercurial branch that this release is on.
 #
-CVS_BRANCH_TAG=HEAD
+HG_BRANCH_TAG=GraphicsMagick-1_3
 
 #
 # Libtool library revision control info
 # See the libtool documentation under the heading "Libtool's versioning
 # system" in order to understand the meaning of these fields
 #
+# current
+#      The most recent interface number that this library implements.
+# revision
+#      The implementation number of the current interface.
+# age
+#      The difference between the newest and oldest interfaces that
+#      this library implements. In other words, the library implements
+#      all the interface numbers in the range from number current -
+#      age to current.
+#
 # Here are a set of rules to help you update your library version
 # information:
 #
@@ -63,20 +86,20 @@ CVS_BRANCH_TAG=HEAD
 #
 # Magick library versioning
 #
-MAGICK_LIBRARY_CURRENT=3
-MAGICK_LIBRARY_REVISION=5
-MAGICK_LIBRARY_AGE=0
+MAGICK_LIBRARY_CURRENT=12
+MAGICK_LIBRARY_REVISION=0
+MAGICK_LIBRARY_AGE=9
 
 #
 # Magick++ library versioning
 #
-MAGICK_PLUS_PLUS_LIBRARY_CURRENT=3
-MAGICK_PLUS_PLUS_LIBRARY_REVISION=1
-MAGICK_PLUS_PLUS_LIBRARY_AGE=0
+MAGICK_PLUS_PLUS_LIBRARY_CURRENT=9
+MAGICK_PLUS_PLUS_LIBRARY_REVISION=0
+MAGICK_PLUS_PLUS_LIBRARY_AGE=6
 
 #
 # Magick Wand library versioning
 #
-MAGICK_WAND_LIBRARY_CURRENT=2
-MAGICK_WAND_LIBRARY_REVISION=1
-MAGICK_WAND_LIBRARY_AGE=0
+MAGICK_WAND_LIBRARY_CURRENT=7
+MAGICK_WAND_LIBRARY_REVISION=0
+MAGICK_WAND_LIBRARY_AGE=5
