@@ -92,6 +92,9 @@ extern WandExport double
 extern WandExport ImageType
   MagickGetImageType(MagickWand *);
 
+extern WandExport ImageType
+  MagickGetImageSavedType(MagickWand *);
+
 extern WandExport InterlaceType
   MagickGetImageInterlaceScheme(MagickWand *);
 
@@ -174,6 +177,7 @@ extern WandExport unsigned int
   MagickEmbossImage(MagickWand *,const double,const double),
   MagickEnhanceImage(MagickWand *),
   MagickEqualizeImage(MagickWand *),
+  MagickExtentImage(MagickWand *,const size_t,const size_t,const ssize_t, const ssize_t),
   MagickFlipImage(MagickWand *),
   MagickFlopImage(MagickWand *),
   MagickFrameImage(MagickWand *,const PixelWand *,const unsigned long,
@@ -249,6 +253,7 @@ extern WandExport unsigned int
   MagickSeparateImageChannel(MagickWand *,const ChannelType),
   MagickSetCompressionQuality(MagickWand *wand,const unsigned long quality),
   MagickSetFilename(MagickWand *,const char *),
+  MagickSetFormat(MagickWand *,const char *),
   MagickSetImage(MagickWand *,const MagickWand *),
   MagickSetImageAttribute(MagickWand *,const char *, const char *),
   MagickSetImageBackgroundColor(MagickWand *,const PixelWand *),
@@ -281,6 +286,7 @@ extern WandExport unsigned int
   MagickSetImageResolution(MagickWand *,const double,const double),
   MagickSetImageScene(MagickWand *,const unsigned long),
   MagickSetImageType(MagickWand *,const ImageType),
+  MagickSetImageSavedType(MagickWand *,const ImageType),
   MagickSetImageUnits(MagickWand *,const ResolutionType),
   MagickSetImageVirtualPixelMethod(MagickWand *,const VirtualPixelMethod),
   MagickSetPassphrase(MagickWand *,const char *),
@@ -313,6 +319,7 @@ extern WandExport unsigned int
   MagickWhiteThresholdImage(MagickWand *,const PixelWand *),
   MagickWriteImage(MagickWand *,const char *),
   MagickWriteImageFile(MagickWand *,FILE *),
+  MagickWriteImagesFile(MagickWand *,FILE *,const unsigned int),
   MagickWriteImages(MagickWand *,const char *,const unsigned int);
 
 extern WandExport unsigned long
